
; ----------------------------------------------------------------------
; FUNCOES DE CONTROLE PARA DISPOSITIVOS I2C                   06/01/2015
;
; STATUS: FUNCIONA
;
; Antes de usar as funcoes desta biblioteca, o modulo MSSP
; deve ser inicializado no modo I2C com a macro 'I2C_CONFIG'.
; Exemplo:
;           I2C_CONFIG  .20000000, .100000, SLEWRATE
;
; Definir no programa os simbolos e registadores abaixo:
; i2c_buff      RES     1
; i2c_timeoutL  RES     1
; i2c_timeoutH  RES     1
;
; NOTAS
;
; 1 - Dispositivos I2C podem ter comportamento especifico, por isso, antes
;     de usar este servico, consulte o manual do dispositivo. Um exemplo de
;     uso deste servico pode ser visto no arquivo "ds1307mssp_services.inc".
;
; SPI I/O Ports:  PORTC<3> --> SCL (Clock)
;                 PORTC<4> --> SDA (Data)
;
; ----------------------------------------------------------------------
;
; ----------------------------------------------------------------------
; MACRO: I2C_CONFIG
;
; Inicializa o modulo MSSP no modo I2C:
;
;      I2C_CONFIG  fosc, bitrate, slewrate
; fosc .................. frequencia de operacao em Hertz
; bitrate ............... taxa de transmissao do I2C em Hertz
; slewrate .............. SLEWRATE -----> com controle de slewrate
;                         NO_SLEWRATE --> sem controle de slewrate
; 
; Exemplo com fosc=20MHZ, bitrate=100KHZ com controle de slewrate:
;               I2C_CONFIG  .20000000, .100000, SLEWRATE
;
; Definir os registradores abaixo no programa:
; i2c_buff      RES   1
; i2c_timeoutH  RES   1           ; time out counter High byte
; i2c_timeoutL  RES   1           ; time out counter Low  byte
;
; ----------------------------------------------------------------------
I2C_CONFIG     MACRO   fosc, bitrate, slewrate
               ifndef SLEWRATE
SLEWRATE       equ     1
               endif
               ifndef NO_SLEWRATE
NO_SLEWRATE    equ     2
               endif

               ifndef _i2c_baudrate
                 variable    _i2c_baudrate
               endif

_i2c_baudrate set   ((fosc/bitrate)/4) - 1

               bsf     TRISC,3                   ; SCL is input
               bsf     TRISC,4                   ; SDA is intput
               movlw   b'00111000'               ; bit 5   - Enable MSSP
                                                 ; bit 4   - Release Clock
                                                 ; bit 3:0 - I2C Master Mode 
               movwf   SSPCON1                   ; estabelece modo I2C
               movlw   _i2c_baudrate             ; pega BAUDRATE
               movwf   SSPADD                    ; especifica o BAUDRATE

          if(slewrate==SLEWRATE)
              bcf     SSPSTAT,SMP                ; Com Slew rate 
          else
              bsf     SSPSTAT,SMP                ; Sem Slew rate 
          endif
               ENDM


I2C_SHUTDOWN   MACRO
               clrf   SSPCON1
               ENDM

; ----------------------------------------------------------------------
; Funcao: i2c_wait_idle
;
; Espera o bus I2c ficar livre.
;
; Entrada: nao ha
;
; Saida  : W=0 --> OK
; ----------------------------------------------------------------------
i2c_idle       movf    SSPCON2, w                ; pega SSPCON2
               andlw   0x1F                      ; check ACKEN/RCEN/PEN/RSEN/SEN
               bnz     i2c_idle                  ; espera...
i2c_idle1      btfsc   SSPSTAT, 2                ; transmissao em andamento???
               goto    i2c_idle1                 ; SIM --> espera terminar
               retlw   .0                        ; NAO --> fim

; ------------------------------------------------------------------------
; Aguarda um ACK do slave
; ------------------------------------------------------------------------
i2c_waitACK    call    i2c_waitMSSP              ; aguarda SSPIF ser ligado
i2c_waitACK1   btfsc   SSPCON2,ACKSTAT           ; espera DS1307 responder com ACK
               goto    i2c_waitACK1              ; fica esperando ACK
               return

; ------------------------------------------------------------------------
; Aguarda uma interrupcao do modulo MSSP
; ------------------------------------------------------------------------
i2c_waitMSSP   btfss   PIR1,SSPIF                ; 
               goto    i2c_waitMSSP              ; 
               bcf     PIR1,SSPIF                ; 
               return                            ; 

; ----------------------------------------------------------------------
; Funcao: i2c_start
;
; Envia condicao de Start ao Slave e espera completar durante 32768
; incursoes (i2c_timeout=32768) o que representa, no minimo, cerca de 
; 6,5ms num microcontrolador rondando a 20MHZ ou 32,7ms a 4MHZ.
;
; Entrada: nao ha
;
; Saida  : W=0x00 --> ok
;          W=0x01 --> timeout
;
; ----------------------------------------------------------------------
i2c_start      
i2c_restart    call    i2c_idle                  ; verifica se o bus I2C esta livre
               clrf    i2c_timeoutH              ; limpa timeout
               clrf    i2c_timeoutL              ; limpa timeout
               bsf     SSPCON2, SEN              ; gera condicao de Start
i2c_start1     btfss   SSPCON2, SEN              ; start ok???
               retlw   .0                        ; SIM --> fim normal
               call    i2c_waitMSSP              ; espera interrupcao do MSSP
               incfsz  i2c_timeoutL,f            ; NAO --> contagem timeout
               goto    i2c_start1                ; espera Start completar
               incf    i2c_timeoutH,f            ; timeout
               btfsc   i2c_timeoutH,8            ; tempo esgotado (32768 * Fosc)???
               retlw   .1                        ; SIM --> erro 1
               goto    i2c_start1                ; NAO --> continua esperando

; ----------------------------------------------------------------------
; Funcao: i2c_stop
;
; Envia condicao de Stop ao Slave e espera completar durante 32768
; incursoes (i2c_timeout=32768) o que representa, no minimo, cerca de 
; 6,5ms num microcontrolador rondando a 20MHZ ou 32,7ms a 4MHZ.
; incursoes (i2c_timeout=1024).
;
; Entrada: nao ha
;
; Saida  : W=0x00 --> ok
;          W=0x02 --> timeout
;
; ----------------------------------------------------------------------
i2c_stop       clrf    i2c_timeoutH              ; limpa timeout
               clrf    i2c_timeoutL              ; limpa timeout
               call    i2c_idle                  ; verifica se o bus I2C esta livre
               bsf     SSPCON2, PEN              ; gera condicao de Stop
               call    i2c_waitMSSP              ; espera interrupcao do MSSP
i2c_stop1      btfss   SSPCON2, PEN              ; start ok???
               retlw   .0                        ; SIM --> fim normal
               incfsz  i2c_timeoutL,f            ; NAO --> contagem timeout
               goto    i2c_stop1                 ; espera Start completar
               incf    i2c_timeoutH,f            ; timeout
               btfsc   i2c_timeoutH,8            ; tempo esgotado (32768 * Fosc)???
               retlw   .2                        ; SIM --> erro 2
               goto    i2c_stop1                 ; NAO --> continua esperando

; ----------------------------------------------------------------------
; Funcao: i2c_write
;
; Envia um byte ao Slave.
;
; Entrada: W contem o valor a ser enviado.
;
; Saida  : W=0 --> ok
;          W=4 --> timeout esperando o buffer esvaziar
;          W=6 --> timeout esperando completar o envio
; ----------------------------------------------------------------------
i2c_put        clrf    i2c_timeoutH              ; limpa timeout
               clrf    i2c_timeoutL              ; limpa timeout
i2c_put1       btfss   SSPSTAT, BF               ; SSPBUF cheio???
               goto    i2c_put2                  ; NAO --> vai enviar
               incfsz  i2c_timeoutL,f            ; SIM --> contagem timeout
               goto    i2c_put1                  ; espera completar
               incf    i2c_timeoutH,f            ; timeout
               btfsc   i2c_timeoutH,8            ; tempo esgotado (32768)???
               retlw   .4                        ; SIM --> erro 4
               goto    i2c_put1                  ; Nao --> continua esperando
i2c_put2       clrf    i2c_timeoutH              ; limpa timeout
               clrf    i2c_timeoutL              ; limpa timeout
               movwf   SSPBUF                    ; envia o dado
i2c_put3       btfss   SSPSTAT, BF               ; SSPBUF vazio???
               retlw   .0                        ; SIM --> fim ok
               incfsz  i2c_timeoutL,f            ; contagem timeout
               goto    i2c_put3                  ; espera completar
               incf    i2c_timeoutH,f            ; timeout
               btfsc   i2c_timeoutH,8            ; tempo esgotado (32768)???
               retlw   .6                        ; SIM --> erro 6
               goto    i2c_put3                  ; Nao --> continua esperando


; ----------------------------------------------------------------------
; Funcao: i2c_read
;
; Recebe um byte do Slave.
;
; Entrada: nao ha
;
; Saida  : i2c_buff contera o dado lido
;          W=0  --> ok
;          W=8  --> timeout esperando o buffer ficar pronto
;          W=10 --> timeout na leitura
;
; ----------------------------------------------------------------------
i2c_get        clrf    i2c_timeoutH              ; limpa timeout
               clrf    i2c_timeoutL              ; limpa timeout
i2c_get1       btfss   SSPSTAT, BF               ; SSPBUF cheio???
               goto    i2c_get2                  ; NAO --> vai receber
               incfsz  i2c_timeoutL,f            ; NAO --> contagem timeout
               goto    i2c_get1                  ; espera completar
               incf    i2c_timeoutH,f            ; timeout
               btfsc   i2c_timeoutH,8            ; tempo esgotado (32768)???
               retlw   .8                        ; SIM --> erro 8
               goto    i2c_get1                  ; Nao --> continua esperando
i2c_get2       bsf     SSPCON2, RCEN             ; modo de recepcao
               clrf    i2c_timeoutH              ; limpa timeout
               clrf    i2c_timeoutL              ; limpa timeout
i2c_get3       btfsc   SSPSTAT, BF               ; SSPBUF vazio???
               goto    i2c_get4                  ; SIM --> espera SSPBUF encher 
               incfsz  i2c_timeoutL,f            ; contagem timeout
               goto    i2c_get3                  ; espera completar
               incf    i2c_timeoutH,f            ; timeout
               btfsc   i2c_timeoutH,8            ; tempo esgotado (32768)???
               retlw   .10                       ; SIM --> erro 10
               goto    i2c_get3                  ; Nao --> continua esperando
i2c_get4       movf    SSPBUF, w                 ; pega o dado que veio em SSPBUF  
               movwf   i2c_buff                  ; salva o dado recebido
               retlw   .0                        ; fim normal

; ----------------------------------------------------------------------
; Funcao: i2c_nack
;
; Envia um NACK ao Slave
;
; Entrada: nao ha
;
; Saida  : W=0 --> ok
;
; ----------------------------------------------------------------------
i2c_nack       bsf     SSPCON2, ACKDT
               bsf     SSPCON2, ACKEN
i2c_nack1      btfsc   SSPCON2, ACKEN
               goto   i2c_nack1
               retlw   .0

; ----------------------------------------------------------------------
; Funcao: i2c_ack
;
; Envia um ACK ao Slave
;
; Entrada: nao ha
;
; Saida  : W=0 --> ok
;
; ----------------------------------------------------------------------
i2c_ack        call    i2c_idle                  ;  verifica se o bus I2C esta livre
               bcf     SSPCON2, ACKDT
               bsf     SSPCON2, ACKEN
i2c_ack1       btfsc   SSPCON2, ACKEN
               goto    i2c_ack1
               call    i2c_waitMSSP              ; 
               retlw   .0

